/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvvideorecorder.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVVideoRecorder>

QVVideoRecorder::~QVVideoRecorder()
	{
	videoFile.close();
	}

bool QVVideoRecorder::linkProperty(QString sourcePropertyName, QVPropertyContainer *destinyContainer, QString destinyPropertyName, LinkType linkType)
	{ return false; }

QVVideoRecorder::QVVideoRecorder(QString name, const QString fileName, const int fps): QVWorker(name), initiated(false), rgbMode(true)
	{
	addProperty< QVImage<uChar,3> >("Input image", inputFlag);
	addProperty< QVImage<uChar,1> >("Input image Y", inputFlag);
	addProperty< QVImage<uChar,1> >("Input image U", inputFlag);
	addProperty< QVImage<uChar,1> >("Input image V", inputFlag);

	addProperty< QString >("Record file name", inputFlag, fileName);
	addProperty< int >("FPS", inputFlag, fps);
	}

bool QVVideoRecorder::linkProperty(QVWorker &worker, const QString rgbImageName)
	{
	worker.linkProperty(rgbImageName, this, "Input image", SynchronousLink);
	rgbMode = true;
	}

bool QVVideoRecorder::linkProperty(QVWorker &worker, const QString yImageName, const QString uImageName, const QString vImageName)
	{
	worker.linkProperty(yImageName, this, "Input image Y", SynchronousLink);
	worker.linkProperty(uImageName, this, "Input image U", SynchronousLink);
	worker.linkProperty(vImageName, this, "Input image V", SynchronousLink);
	rgbMode = false;
	}

void QVVideoRecorder::iterate()
	{
	if (!initiated)
		{
		int cols, rows;

		if (rgbMode)
			{
			const QVImage<uChar, 3> imageRGB = getPropertyValue< QVImage<uChar, 3> >("Input image");
			cols = imageRGB.getCols();
			rows = imageRGB.getRows();
			}
		else	{
			const QVImage<uChar, 1> imageY = getPropertyValue< QVImage<uChar, 1> >("Input image Y");
			cols = imageY.getCols();
			rows = imageY.getRows();
			}
		if (cols > 1 && rows > 1)
			{
			videoFile.setFileName(getPropertyValue<QString>("Record file name"));
			videoFile.open(QIODevice::WriteOnly|QIODevice::Truncate);
			writeYUV4MPEG2Header(videoFile, cols, rows, getPropertyValue<int>("FPS"));
			initiated = true;
			}
		}
	if (rgbMode)
		{
		const QVImage<uChar, 3> imageRGB = getPropertyValue< QVImage<uChar, 3> >("Input image");
		writeYUV4MPEG2Frame(videoFile, imageRGB);
		}
	else	{
		const QVImage<uChar, 1> imageY = getPropertyValue< QVImage<uChar, 1> >("Input image Y");
		const QVImage<uChar, 1> imageU = getPropertyValue< QVImage<uChar, 1> >("Input image U");
		const QVImage<uChar, 1> imageV = getPropertyValue< QVImage<uChar, 1> >("Input image V");
		writeYUV4MPEG2Frame(videoFile, imageY, imageU, imageV);
		}
	}
