/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvmath.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVMATH_H
#define QVMATH_H

#include <iostream>
#include <cblas.h>
#include <QPointF>

#include <qvdefines.h>

/// @todo document
const int qvFactorial(const int n);

/*! @brief Number of combinations of a fixed size over the elements of a set

Given a set size of <i>n</i>, this function returns the number of subsets of size <i>k</i>
within that set.

\f$ qvCombination(n,k) = {n \choose k} = \frac{n!}{k!(n-k)!} \f$
@ingroup qvmath
*/
const double qvCombination(const int setRange, const int subsetRange);

/// @todo document
/// @ingroup qvmath
const double qvAngle(const QPointF &);

/// @todo document
/// @ingroup qvmath
const double qvClockWiseAngle(const QPointF &, const QPointF &);

/// @todo document
/// @ingroup qvmath
const int qvRandom(const int minValue, const int maxValue);

/// @todo document
/// @ingroup qvmath
std::ostream& operator << ( std::ostream &os, const QPointF &point );

/// @todo document
/// @ingroup qvmath
double norm2(const QPointF &p);

#endif
